# File : jftI2C.py
# 20170308 Created: version 0.0.1 "Initial version"
# 20170703 Adapted for 64 bits version
# 20170920 Separate I2C library

from ctypes import*

# assume location of dll in search path

import sys
if (sys.maxsize > 2**32):
    jftserial1 = cdll.LoadLibrary("jftserial_64")
else:
    jftserial1 = cdll.LoadLibrary("jftserial")

import jft
import array

# adapt for version 3.4
def to_char_p(a):
    return c_char_p(a.encode('utf-8'))

def ProcessReturnValue(retVal):
    if(retVal == 0):
        pass
    else:
        jft.PrintErrorTraceback()    
        jft.PrintMessage(retVal)
        sys.exit(2)

#////////////////////////////////////////////////////////////////////////////////
#// I2C
#////////////////////////////////////////////////////////////////////////////////

#declaration of result types of all dll functions:
jftserial1.JftI2c_SlaveDiscovery.restype = POINTER(c_ushort) #
# all other dll fuctions return c_int, which is default for Python

I2c_DataReturnMode = 'list'
def GetResultMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetParam(to_char_p(I2cInterface), to_char_p("DataReturnMode"), byref(ModeRP) )
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    resultMode = ['list','int_LF','int_MF'][mode]
    return resultMode

I2C_helpText = "****************  I2C Help text  *******************\n"

def Help():
    print(I2C_helpText)


I2C_helpText += "SetResultMode sets the output format of READ.\n" \
                "Parameters:\n" \
                " resultMode    'int_LF', 'int_MF' or 'list'.\n" \
                "NOTE: prints error string if resultmode is not listed above.\n" \
                "   \n"

#Receive Multiple bytes, byte order:
#"LSByte is received first" 0x02DC6C00 is received as:  0x00, 0x6Ch, 0xDC, 0x02
#"MSByte is received first" 0x02DC6C00 is received as:  0x02, 0xDC, 0x6Ch, 0x00
def SetResultMode(I2cInterface, resultMode):
    if resultMode.lower() == 'list':
        DataReturnMode = 'list'
        jftserial1.JftI2c_SetParam(to_char_p(I2cInterface),to_char_p("DataReturnMode"),0)
    elif resultMode.lower() == 'int_lf':
        DataReturnMode = 'int_LF'
        jftserial1.JftI2c_SetParam(to_char_p(I2cInterface),to_char_p("DataReturnMode"),1)
    elif resultMode.lower() == 'int_mf':
        DataReturnMode = 'int_MF'
        jftserial1.JftI2c_SetParam(to_char_p(I2cInterface),to_char_p("DataReturnMode"),2)
    else:
        jft.printToStdErr("Error, ResultMode must be 'list' or 'int_LF', or 'int_MF', not ",resultMode)
        DataReturnMode = ''
	

Ack = 0

# ReturnPointers
StatusRP = c_int()
ExitStatusRP = c_int()
FrequencyRP = c_long()
StateRP = c_uint()

I2C_helpText += "CreateInterface creates an I2C interface from 2 pins of the JFT project.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " SclPin        A JFT pin name that will be used by this interface as SCL pin\n" \
                " SdaPin        A JFT pin name that will be used by this interface as SDA pin\n" \
                " Dependent on the initial ProVision application setup and VDF file, \n" \
                " the pin name string may have one of following formats:\n" \
                "               'U12.IOR6' (single board, pin names) \n" \
                "               'U12.R6' (single board, pin numbers) \n" \
                "               'jt2156_1.U12.IOR6' (multi board, pin names) \n" \
                "               'jt2156_1.U12.R6' (multi board, pin numbers) \n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "NOTE: To be able to use this I2C interface a device address of an I2C device\n" \
                "      must be set to this I2C interface with the function SetDevice().\n" \
                "   \n"

def CreateInterface(I2cInterface, SclPin, SdaPin):
    retVal = jftserial1.JftI2c_CreateInterface( to_char_p(I2cInterface), to_char_p(SclPin), to_char_p(SdaPin) )
    jft.ProcessReturnValue(retVal)
    jftserial1.JftI2c_SetParam(to_char_p(I2cInterface),to_char_p("DataReturnMode"),0)
    return retVal


I2C_helpText += "SetDevice sets the device to be used on the I2C interface by specifying\n" \
                "the I2C device address of that device. To use another device on this I2C\n" \
                "interface call this function again with the device address of that I2C\n" \
                "device.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " DeviceAddress  The device address of the I2C device to be used.\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetDevice(I2cInterface, DeviceAddress):
    retVal = jftserial1.JftI2c_SetDevice(to_char_p(I2cInterface), c_ushort(DeviceAddress) )
    jft.ProcessReturnValue(retVal)
    return retVal

I2C_helpText += "GetDevice returns the current slave device address that has been set\n" \
                "with the function SetDevice().\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " DeviceAddress  The device address of the I2C device that has been set with the\n" \
                "Return values:\n" \
                " The address of the current device that is connected to the given I2C interface\n" \
                "               Any system error interrupts the execution and exits with an error message\n" \
                "   \n"

def GetDevice(I2cInterface):
    DeviceAddressRP = c_ushort()
    retVal = jftserial1.JftI2c_GetDevice(to_char_p(I2cInterface), byref(DeviceAddressRP) )
    jft.ProcessReturnValue(retVal)
    DeviceAddress = DeviceAddressRP.value
    return DeviceAddress


I2C_helpText += "SetSclMode sets the mode of the SCL pin of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The mode of the SCL pin, i.e.: how it sends a '1'\n" \
                "               Valid range:\n" \
                "               0 (open collector / open drain) (default) \n" \
                "               1 (push pull)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                " 2             System error\n" \
                "   \n"

def SetSclMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetSclMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetSclMode returns the mode of the SCL pin of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The mode of the SCL pin\n" \
                "   \n"

def GetSclMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetSclMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetSdaMode sets the mode of the SDA pin of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The mode of the SDA pin, i.e.: how it sends a '1'\n" \
                "               Valid range:\n" \
                "               0 (open collector / open drain) (default) \n" \
                "               1 (push pull)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetSdaMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetSdaMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetSdaMode returns the mode of the SDA pin of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The mode of the SDA pin\n" \
                "   \n"

def GetSdaMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetSdaMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetBitMode sets the bit mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The bit mode to be used\n" \
                "               Valid range:\n" \
                "               0 (LSB first) \n" \
                "               1 (MSB first) (default)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetBitMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetBitMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetBitMode returns the bit mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The bit mode\n" \
                "   \n"

def GetBitMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetBitMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetOptimizeMode sets the optimize mode of the specified I2C interface. If the\n" \
                "optimize mode is 'ON' (default) controller acccess will be reduced as much as\n" \
                "possible in terms of skipping DriveLow of signal is already low.\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The optimize mode to be used\n" \
                "               Valid range:\n" \
                "               0 (optimize mode off) \n" \
                "               1 (optimize mode on) (default)\n" \
                "               MultiScan ON \n" \
                "               2 (optimize mode off) \n" \
                "               3 (optimize mode on) (default)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetOptimizeMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetOptimizeMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetOptimizeMode returns the optimize mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The optimize mode\n" \
                "   \n"

def GetOptimizeMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetOptimizeMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetRepeatedStartMode sets the repeated start mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The repeated start mode to be used\n" \
                "               Valid range:\n" \
                "               0 (repeated start mode off -> replace a repeated start\n" \
                "                                             condition with a stop condition\n" \
                "                                             followed by a start condition) \n" \
                "               1 (repeated start mode on) (default)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetRepeatedStartMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetRepeatedStartMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetRepeatedStartMode returns the repeated start mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The repeated start mode\n" \
                "   \n"

def GetRepeatedStartMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetRepeatedStartMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetDeviceAddressMode sets the address mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The slave addressing mode to be used\n" \
                "               Valid range:\n" \
                "               7  (use 7 bit addressing mode) (default) \n" \
                "               10 (use 10 bit addressing mode)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetDeviceAddressMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetDeviceAddressMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetDeviceAddressMode returns the address mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The address mode\n" \
                "   \n"

def GetDeviceAddressMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetDeviceAddressMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode

I2C_helpText += "SetRegisterAddressMode sets the register addressing mode to be used in I2C devices\n" \
                "connected to the I2C interface referenced by the I2cInterface parameter. \n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The register addressing mode to be used in the I2C device\n" \
                "               Valid range:\n" \
                "                0 (do not use I2C device register addressing mode) \n" \
                "                8 (use 8 bit I2C device register addressing mode) (default)\n" \
                "               16 (use 16 bit I2C device register addressing mode)\n" \
                "               24 (use 24 bit I2C device register addressing mode)\n" \
                "               32 (use 32 bit I2C device register addressing mode)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetRegisterAddressMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetRegisterAddressMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetRegisterAddressMode returns the register addressing mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The register addressing mode\n" \
                "   \n"

def GetRegisterAddressMode(I2cInterface):   
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetRegisterAddressMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetStartProcedureMode sets the start procedure mode to be used on the I2C interface\n" \
                "referenced by the I2cInterface parameter. \n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " mode          The start procedure mode to be used on the I2C bus\n" \
                "               Valid range:\n" \
                "               0 (use start condition only) (default)\n" \
                "               1 (use start condition and start byte)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetStartProcedureMode(I2cInterface, mode):
    retVal = jftserial1.JftI2c_SetStartProcedureMode(to_char_p(I2cInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetStartProcedureMode returns the start procedure mode of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The start procedure mode\n" \
                "   \n"

def GetStartProcedureMode(I2cInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftI2c_GetStartProcedureMode(to_char_p(I2cInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


I2C_helpText += "SetFirstRegisterAddress sets the starting register address for reading (with the\n" \
                "ReadRegisters() function) or writing (with the WriteRegisters\n" \
                "function). This address will be used on all I2C devices connected to the I2C\n" \
                "interface referenced by the I2cInterface parameter. This address will be\n" \
                "ignored if the register address mode has been set to 0.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " RegisterAddress  The register address to start reading and writing to an I2C\n" \
                "                  device on the I2C bus.\n" \
                "                  Valid range:\n" \
                "                   ignored                 (register address mode  0) \n" \
                "                         0x00 -       0xFF (register address mode  8)\n" \
                "                       0x0000 -     0xFFFF (register address mode 16)\n" \
                "                     0x000000 -   0xFFFFFF (register address mode 24)\n" \
                "                   0x00000000 - 0xFFFFFFFF (register address mode 32)\n" \
                "                  Default:\n" \
                "                   0x00\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetFirstRegisterAddress(I2cInterface,RegisterAddress):
    if (type(RegisterAddress) != int):
        RegisterAddressInt = int("0b"+RegisterAddress , 2)
    else:
        RegisterAddressInt = RegisterAddress

    retVal = jftserial1.JftI2c_SetFirstRegisterAddress(to_char_p(I2cInterface),c_ulong(RegisterAddressInt) )
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "GetFirstRegisterAddress returns the starting register address of the specified I2C interface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The starting register address\n" \
                "   \n"

def GetFirstRegisterAddress(I2cInterface):
    RegisterAddressRP = c_ulong()
    retVal = jftserial1.JftI2c_GetFirstRegisterAddress(to_char_p(I2cInterface), byref(RegisterAddressRP))
    jft.ProcessReturnValue(retVal)
    RegisterAddress = RegisterAddressRP.value
    return RegisterAddress


I2C_helpText += "WriteRegisters writes ByteCnt bytes starting at register FirstRegisterAddress on\n" \
                "the I2C device with address DeviceAddress on the I2C bus with I2C interface\n" \
                "I2cInterface. It also reads the Ack of the addressed slave device.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " writeData     The data to be written to the I2C device. \n" \
                "               Numbers (127, 0xE3F0, 0b11001101) or binary string ('11001101') are allowed.\n" \
                " ByteCnt       The number of bytes that will be written to the I2C device.\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "NOTE: use GetAck() to find out if the addressed device responded with Ack immediately.\n" \
                "   \n"

def WriteRegisters(I2cInterface, writeData, ByteCnt):
    global Ack
    list_n = []
    
    if (ByteCnt==0):
        jft.printToStdErr("Error, ByteCnt must be > 0")
        return 0

    for n in range(0,ByteCnt): list_n.append(0)

    if (type(writeData) == list) or (type(writeData) == tuple):
        writeDataSize = len(writeData)
        if(writeDataSize > ByteCnt):
            jft.printToStdErr("Error, Size writeData larger than ByteCnt parameter")
            return 0
        for n in range(0,writeDataSize): list_n[n] = writeData[n]
    else:
        if (type(writeData) != int):
            writeDataInt = int("0b"+writeData , 2)
        else:
            writeDataInt = writeData
        
        if (writeDataInt > 0xFF):
            jft.printToStdErr("Error, Size writeData larger than one Byte, use a list")
            return 0
        
        list_n[0] = writeDataInt

    ByteOutArray = array.array('B',list_n) # 'B' = unsigned char
    AckRP = c_ubyte()
    retVal = jftserial1.JftI2c_WriteRegisters(to_char_p(I2cInterface), ByteOutArray.buffer_info()[0], c_int(ByteCnt), byref(AckRP))
    jft.ProcessReturnValue(retVal)
    Ack = AckRP.value
    return retVal


I2C_helpText += "ReadRegisters reads ByteCnt bytes starting at register FirstRegAddress on\n" \
                "the I2C device with address DeviceAddress on the I2C bus with I2C interface\n" \
                "I2cInterface. It also reads the Ack of the addressed slave device.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                " ByteCnt          The number of bytes that will be read from the I2C device.\n" \
                "Return values:\n" \
                " A Python integer containing the read value. \n" \
                "               Any system error interrupts the execution and exits with an error message\n" \
                "NOTE1: Use SetReturnMode( ) to alter the output format.\n" \
                "NOTE2: Use GetAck() to find out if the addressed device responded with Ack immediately.\n" \
                "   \n"

def ReadRegisters(I2cInterface, ByteCnt):
    global Ack
    I2c_DataReturnMode = GetResultMode(I2cInterface)
    list_n = []
    for n in range(0,ByteCnt): list_n.append(0)

    ByteInArray = array.array('B',list_n) # 'B' = unsigned char

    AckRP = c_ubyte()
    retVal = jftserial1.JftI2c_ReadRegisters(to_char_p(I2cInterface), c_void_p(ByteInArray.buffer_info()[0]), c_int(ByteCnt), byref(AckRP) )
    jft.ProcessReturnValue(retVal)
    Ack = AckRP.value

    resultInt_LF = 0
    resultInt_MF = 0
    uByteList = ByteInArray.tolist()
    
    if I2c_DataReturnMode == 'int_MF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_MF = resultInt_MF*256 + section
        return resultInt_MF
    elif I2c_DataReturnMode == 'int_LF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_LF = resultInt_LF + section*(256**(i))
        return resultInt_LF
    elif I2c_DataReturnMode == 'list':
        return uByteList
    else:
        jft.printToStdErr("Error, ResultMode must be 'list' or 'int_LF', or 'int_MF', not ",I2c_DataReturnMode)
        return []


I2C_helpText += "GetAck returns acknowledgement by the slave immediately after the last call\n" \
                "to ReadRegisters() or WriteRegisters()\n" \
                "Return values:\n" \
                " Ack              After successful exection of this function this variable\n" \
                "                  will contain 1 if all written bytes by the master were\n" \
                "                  acknowledged by the slave otherwise it will contain 0.\n" \
                "NOTE: GetAck() does not access the I2C bus to read the Ack. It produces the Ack of the last call.\n" \
                "   \n"

def GetAck():
    global Ack
    return Ack


I2C_helpText += "SoftwareReset attempts to reset all devices on the I2C bus (referenced by a\n" \
                "valid I2cInterface) by writing to general I2C call address (0x00) the\n" \
                "software reset command (0x06). All I2C devices on the specified I2C bus that\n" \
                "implement this functionality will reset themselves.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SoftwareReset(I2cInterface):
    retVal = jftserial1.JftI2c_SoftwareReset(to_char_p(I2cInterface))
    jft.ProcessReturnValue(retVal)
    return retVal

I2C_helpText += "ClearBus attempts to recover an I2C bus (referenced by a valid \n" \
                "I2CInterface) from a SDA pin stuck at 0 by toggling the SCL and SDA pins.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def ClearBus(I2cInterface):
    retVal = jftserial1.JftI2c_ClearBus(to_char_p(I2cInterface))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "SlaveDiscovery will discover all connected I2C slave devices on the I2C bus\n" \
                "referenced by a valid I2cInterface.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return Value       Returns a list containing addresses of discovered I2C slave devices.\n" \
                "                   Any error interrupts the execution and exits with an error message\n" \
                "   \n"

def SlaveDiscovery(I2cInterface):
    DiscoveredSlavesCountRP = c_int()
    RetValRP = c_int()
    DiscoveredSlavesArray = jftserial1.JftI2c_SlaveDiscovery(to_char_p(I2cInterface), byref(DiscoveredSlavesCountRP), byref(RetValRP) )
    retVal = RetValRP.value
    jft.ProcessReturnValue(retVal)
    DiscoveredSlavesCount = DiscoveredSlavesCountRP.value
    SlaveList = []
    for n in range(0,DiscoveredSlavesCount): 
        SlaveList.append(DiscoveredSlavesArray[n])
    return SlaveList


I2C_helpText += "RemoveInterface removes the I2C Interface and all allocated resources of the\n" \
                "I2C Interface previously created with a successful call to the\n" \
                "JftIc2_CreateInterface() function.\n" \
                "Parameters:\n" \
                " I2cInterface  The name of any I2cInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def RemoveInterface(I2cInterface):
    retVal = jftserial1.JftI2c_RemoveInterface(to_char_p(I2cInterface))
    jft.ProcessReturnValue(retVal)
    return retVal


I2C_helpText += "****************  End of I2C Help text  *******************\n"


























