# File : jftSPI.py
# 20170308 Created: version 0.0.1 "Initial version"
# 20170703 Adapted for 64 bits version
# 20170920 Separate SPI library
# 20171108 Organisation of Input data and Output data

from ctypes import*

# assume location of dll in search path

import sys
if (sys.maxsize > 2**32):
    jftserial1 = cdll.LoadLibrary("jftserial_64")
else:
    jftserial1 = cdll.LoadLibrary("jftserial")

import jft
import array

# adapt for version 3.4
def to_char_p(a):
    return c_char_p(a.encode('utf-8'))

def ProcessReturnValue(retVal):
    if(retVal == 0):
        pass
    else:
        jft.PrintErrorTraceback()
        jft.PrintMessage(retVal)
        sys.exit(2)

#////////////////////////////////////////////////////////////////////////////////
#// SPI
#////////////////////////////////////////////////////////////////////////////////

#declaration of result types of all dll functions:
jftserial1.JftSpi_GetDevice.restype = c_char_p      #(char *SpiInterface, int *retValRP)
# all other dll fuctions return c_int, which is default for Python

SPI_helpText = "****************  SPI Help text  *******************\n"

def Help():
    print(SPI_helpText)


SPI_helpText += "SetResultMode sets the output format of READ and READ/WRITE.\n" \
                "Parameters:\n" \
                " resultMode    'int_LF' int_MF or 'list'. (case insensitive)\n" \
                "NOTE: prints error string if resultmode is not listed above.\n" \
                "   \n"

#Receive Multiple bytes, byte order:
#"LSByte is received first" 0x02DC6C00 is received as:  0x00, 0x6Ch, 0xDC, 0x02
#"MSByte is received first" 0x02DC6C00 is received as:  0x02, 0xDC, 0x6Ch, 0x00
def SetResultMode(SpiInterface, resultMode):
    if resultMode.lower() == 'list':
        DataReturnMode = 'list'
        jftserial1.JftSpi_SetParam(to_char_p(SpiInterface),to_char_p("DataReturnMode"),0)
    elif resultMode.lower() == 'int_lf':
        DataReturnMode = 'int_LF'
        jftserial1.JftSpi_SetParam(to_char_p(SpiInterface),to_char_p("DataReturnMode"),1)
    elif resultMode.lower() == 'int_mf':
        DataReturnMode = 'int_MF'
        jftserial1.JftSpi_SetParam(to_char_p(SpiInterface),to_char_p("DataReturnMode"),2)
    else:
        jft.printToStdErr("Error, ResultMode must be 'list' or 'int_LF', or 'int_MF', not ",resultMode)
        DataReturnMode = ''

def GetResultMode(SpiInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetParam(to_char_p(SpiInterface), to_char_p("DataReturnMode"), byref(ModeRP) )
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    resultMode = ['list','int_LF','int_MF'][mode]
    return resultMode

        
SPI_helpText += "CreateInterface creates an SPI interface from 3 pins of the JFT project.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " SckPin        A JFT pin name that will be used by this interface as SCK pin\n" \
                " MisoPin       A JFT pin name that will be used by this interface as MISO\n" \
                "               pin or an empty string if no MISO pin is available\n" \
                "               (the interface will be write-only).\n" \
                " MosiPin       A JFT pin name that will be used by this interface as MOSI\n" \
                "               pin or an empty string if no MOSI pin is available\n" \
                "               (the interface will be read-only).\n" \
                " Dependent on the initial ProVision application setup and VDF file, \n" \
                " the pin name string may have one of following formats:\n" \
                "               'U12.IOW2' (single board, pin names) \n" \
                "               'U12.W2' (single board, pin numbers) \n" \
                "               'jt2156_1.U12.IOW2' (multi board, pin names) \n" \
                "               'jt2156_1.U12.W2' (multi board, pin numbers) \n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "NOTE: To be able to use this SPI interface a SPI device must be connected to\n" \
                "      this interface with the function SetDevice().\n" \
                "   \n"

def CreateInterface( SpiInterface, SckPin, MisoPin, MosiPin ):
    retVal = jftserial1.JftSpi_CreateInterface(to_char_p(SpiInterface), to_char_p(SckPin), to_char_p(MisoPin), to_char_p(MosiPin) )
    jft.ProcessReturnValue(retVal)
    jftserial1.JftSpi_SetParam(to_char_p(SpiInterface),to_char_p("DataReturnMode"),0)
    return retVal


SPI_helpText += "SetDevice sets the device to be used on the SPI interface by specifying\n" \
                "the SS (Slave Select) pin of that device. To use another device on a SPI\n" \
                "interface call this function again with another SsPin name.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " SsPin         A JFT pin name that should be connected to the SS pin of the\n" \
                "               SPI device to be used  \n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetDevice(SpiInterface, SsPin):
    retVal = jftserial1.JftSpi_SetDevice(to_char_p(SpiInterface), to_char_p(SsPin))
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "GetDevice returns the SS (Slave Select) pin name of the device that will be\n" \
                "used on the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The name of the Slave Select pin that was activated by SetDevice().\n" \
                "               Any system error interrupts the execution and exits with an error message\n" \
                "   \n"

def GetDevice(SpiInterface):
    retValRP = c_int()
    SsPinBuf = jftserial1.JftSpi_GetDevice(to_char_p(SpiInterface), byref(retValRP) )
    retVal = retValRP.value
    jft.ProcessReturnValue(retVal)
    return SsPinBuf.decode("utf-8")


SPI_helpText += "SetCpolMode sets the CPOL mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " mode          The CPOL mode to be used\n" \
                "               Valid range:\n" \
                "               0 (CPOL mode 0, SCK starts low) (default) \n" \
                "               1 (CPOL mode 1, SCK starts high)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetCpolMode( SpiInterface, mode ):
    retVal = jftserial1.JftSpi_SetCpolMode(to_char_p(SpiInterface), c_int(mode) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "GetCpolMode returns the CPOL mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The CPOL mode\n" \
                "   \n"

def GetCpolMode( SpiInterface ):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetCpolMode(to_char_p(SpiInterface), byref(ModeRP) )
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


SPI_helpText += "SetCphaMode sets the CPHA mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " mode          The CPHA mode to be used\n" \
                "               Valid range:\n" \
                "               0 (CPHA mode 0) (default) \n" \
                "               1 (CPHA mode 1)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetCphaMode(SpiInterface, mode ):
    retVal = jftserial1.JftSpi_SetCphaMode(to_char_p(SpiInterface), c_int(mode) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "GetCphaMode returns the CPHA mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The CPHA mode\n" \
                "   \n"

def GetCphaMode( SpiInterface ):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetCphaMode(to_char_p(SpiInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


SPI_helpText += "SetSsMode sets the mode of the SS pin of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " mode          The SS pin mode to be used\n" \
                "               Valid range:\n" \
                "               0 (SS pin active low) (default) \n" \
                "               1 (SS pin active high)\n" \
                "               2 (SS pin low pulse 1 on start)\n" \
                "               3 (SS pin high pulse 0 on start)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetSsMode(SpiInterface, mode ):
    retVal = jftserial1.JftSpi_SetSsMode(to_char_p(SpiInterface), c_int(mode) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "GetSsMode returns the mode of the SlaveSelect pin of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The mode of the SlaveSelect pin\n" \
                "   \n"

def GetSsMode( SpiInterface ):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetSsMode(to_char_p(SpiInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


SPI_helpText += "SetBitMode sets the bit mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " mode          The bit mode to be used\n" \
                "               Valid range:\n" \
                "               0 (LSB first)\n" \
                "               1 (MSB first) (default)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetBitMode( SpiInterface, mode ):
    retVal = jftserial1.JftSpi_SetBitMode(to_char_p(SpiInterface), c_int(mode) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "GetBitMode returns the bit mode of the specified SPI interface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " The bit mode\n" \
                "   \n"

def GetBitMode( SpiInterface ):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetBitMode(to_char_p(SpiInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


SPI_helpText += "SetOptimizeMode sets the optimize mode of the specified SPI interface. If the\n" \
                "optimize mode is 'ON' (default) controller acccess will be reduced as much as\n" \
                "possible in terms of skipping DriveLow of signal is already low.\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " mode          The optimize mode to be used\n" \
                "               Valid range:\n" \
                "               0 (optimize mode off) \n" \
                "               1 (optimize mode on) (default)\n" \
                "               MultiScan ON \n" \
                "               2 (optimize mode off) \n" \
                "               3 (optimize mode on) (default)\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def SetOptimizeMode(SpiInterface, mode):
    retVal = jftserial1.JftSpi_SetOptimizeMode(to_char_p(SpiInterface), c_int(mode))
    jft.ProcessReturnValue(retVal)
    return retVal

def GetOptimizeMode(SpiInterface):
    ModeRP = c_int()
    retVal = jftserial1.JftSpi_GetOptimizeMode(to_char_p(SpiInterface), byref(ModeRP))
    jft.ProcessReturnValue(retVal)
    mode = ModeRP.value
    return mode


SPI_helpText += "InitInterface initializes the specfied SPI interface for use with the SPI\n" \
                "read and write functions.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "NOTE: Use this function after mode settings.\n" \
                "   \n"

def InitInterface( SpiInterface ):
    retVal = jftserial1.JftSpi_InitInterface(to_char_p(SpiInterface) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "Write writes BitCnt bits on the SPI interface SpiInterface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " writeData     The data to be written to the SPI interface (MOSI). \n" \
                "               A list of bytes, Numbers (127, 0xE3F0, 0b11001101) or  \n" \
                "               binary string ('11001101') are allowed.\n" \
                " BitCnt        The number of bits that will be written to the SPI interface.\n" \
                "               If the BitCnt is not a multiple of 8 the least significant\n" \
                "               bits of the last byte will be written (in the order as\n" \
                "               specified by the SetBitMode() function) \n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def Write( SpiInterface, writeData, BitCnt ):
    ByteCnt = (BitCnt+7)//8
    
    if (ByteCnt==0):
        jft.printToStdErr("Error, BitCnt must be > 0")
        return 0

    writeDataSize = 0

    list_n = []
    for n in range(0,ByteCnt): list_n.append(0)

    if (type(writeData) == list) or (type(writeData) == tuple):
        writeDataSize = len(writeData)
        if(writeDataSize > ByteCnt):
            jft.printToStdErr("Error, Size writeData larger than ByteCnt parameter")
            return 0
        for n in range(0,writeDataSize): list_n[n] = writeData[n]
    else:
        if (type(writeData) != int):
            writeDataInt = int("0b"+writeData , 2)
        else:
            writeDataInt = writeData
        
        import math
        if(writeDataInt == 0):
            writeDataSize = 1
        else:
            writeDataSize = math.floor(math.log(writeDataInt,256))+1
        
        if(writeDataSize > ByteCnt):
            jft.printToStdErr("Error, Size writeData larger than ByteCnt parameter")
            return 0

        if(GetBitMode(SpiInterface)== 0):
            for i in range(0,ByteCnt):
                section = writeDataInt % 256
                list_n[i] = section
                writeDataInt = writeDataInt//256 # right shift 8 bits 
        else:
            for i in range(0,writeDataSize):
                section = writeDataInt % 256
                list_n[writeDataSize-i-1] = section
                writeDataInt = writeDataInt//256 # right shift 8 bits 

    ByteOutArray = array.array('B',list_n) # 'B' = unsigned char

    retVal = jftserial1.JftSpi_Write(to_char_p(SpiInterface), c_void_p(ByteOutArray.buffer_info()[0]), c_int(BitCnt) )
    jft.ProcessReturnValue(retVal)
    return retVal


SPI_helpText += "Read reads BitCnt bits from the SPI interface SpiInterface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " ByteArray     The data buffer that will contain the bits read from the SPI\n" \
                "               interface after successful execution of this function. This\n" \
                "               array should have at least (BitCnt + 7) / 8 elements.\n" \
                " BitCnt        The number of bits that will be read from the SPI interface.\n" \
                "               If the BitCnt is not a multiple of 8 only the least\n" \
                "               significant bits of the last byte will be valid. \n" \
                "Return values:\n" \
                " A Python list with bytes received, or an integer containing the read value. \n" \
                "NOTE: Use SetReturnMode( ) to alter the output format.\n" \
                "               Any error interrupts the execution and exits with an error message\n" \
                "   \n"
#Receive Multiple bytes, byte order:
#"Little Endian" 0x02DC6C00 is received as:  0x00, 0x6Ch, 0xDC, 0x02
#"Big Endian" 0x02DC6C00 is received as:     0x02, 0xDC, 0x6Ch, 0x00

def Read( SpiInterface, BitCnt ):
    Spi_DataReturnMode = GetResultMode(SpiInterface)
    ByteCnt = (BitCnt+7)//8

    list_n = []
    for n in range(0,ByteCnt): list_n.append(0)

    ByteInArray = array.array('B',list_n) # 'B' = unsigned char

    retVal = jftserial1.JftSpi_Read(to_char_p(SpiInterface), c_void_p(ByteInArray.buffer_info()[0]), c_int(BitCnt) )
    jft.ProcessReturnValue(retVal)

    resultInt_LF = 0
    resultInt_MF = 0
    uByteList = ByteInArray.tolist()
    
    if Spi_DataReturnMode == 'int_MF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_MF = resultInt_MF*256 + section
        return resultInt_MF
    elif Spi_DataReturnMode == 'int_LF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_LF = resultInt_LF + section*(256**(i))
        return resultInt_LF
    elif Spi_DataReturnMode == 'list':
        return uByteList
    else:
        jft.printToStdErr("Error, ResultMode must be 'list' or 'int_LF', or 'int_MF', not ",Spi_DataReturnMode)
        return []


SPI_helpText += "WriteRead writes and reads BitCnt bits to / from the SPI interface\n" \
                "SpiInterface.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                " writeData     The data to be written to the SPI interface (MOSI). \n" \
                "               A list of bytes, Numbers (127, 0xE3F0, 0b11001101) or  \n" \
                "               binary string ('11001101') are allowed.\n" \
                " BitCnt        The number of bits that will be written to / read from the SPI\n" \
                "               interface. If the BitCnt is not a multiple of 8 only the least\n" \
                "               significant bits of the last byte will be used / valid . \n" \
                "Return values:\n" \
                " A Python list with bytes received, or an integer containing the read value. \n" \
                "NOTE: Use SetReturnMode( ) to alter the output format.\n" \
                "               Any error interrupts the execution and exits with an error message\n" \
                "   \n"

def WriteRead(SpiInterface, writeData, BitCnt ):
    Spi_DataReturnMode = GetResultMode(SpiInterface)
    ByteCnt = (BitCnt+7)//8

    if (ByteCnt==0):
        jft.printToStdErr("Error, BitCnt must be > 0")
        return 0

    writeDataSize = 0

    list_n = []
    for n in range(0,ByteCnt): list_n.append(0)

    if (type(writeData) == list) or (type(writeData) == tuple):
        writeDataSize = len(writeData)
        if(writeDataSize > ByteCnt):
            jft.printToStdErr("Error, Size writeData larger than ByteCnt parameter")
            return 0
        for n in range(0,writeDataSize): list_n[n] = writeData[n]
    else:
        if (type(writeData) != int):
            writeDataInt = int("0b"+writeData , 2)
        else:
            writeDataInt = writeData
        
        import math
        if(writeDataInt == 0):
            writeDataSize = 1
        else:
            writeDataSize = math.floor(math.log(writeDataInt,256))+1
        
        if(writeDataSize > ByteCnt):
            jft.printToStdErr("Error, Size writeData larger than ByteCnt parameter")
            return 0
        if(GetBitMode(SpiInterface)== 0):
            for i in range(0,ByteCnt):
                section = writeDataInt % 256
                list_n[i] = section
                writeDataInt = writeDataInt//256 # right shift 8 bits 
        else:
            for i in range(0,writeDataSize):
                section = writeDataInt % 256
                list_n[writeDataSize-i-1] = section
                writeDataInt = writeDataInt//256 # right shift 8 bits 
            
    ByteOutArray = array.array('B',list_n) # 'B' = unsigned char
    ByteInArray = array.array('B',list_n) # 'B' = unsigned char

    retVal = jftserial1.JftSpi_WriteRead(to_char_p(SpiInterface), c_void_p(ByteOutArray.buffer_info()[0]), c_void_p(ByteInArray.buffer_info()[0]), c_int(BitCnt) )
    jft.ProcessReturnValue(retVal)

    resultInt_LF = 0
    resultInt_MF = 0
    uByteList = ByteInArray.tolist()
    
    if Spi_DataReturnMode == 'int_MF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_MF = resultInt_MF*256 + section
        return resultInt_MF
    elif Spi_DataReturnMode == 'int_LF':
        for i in range(0,ByteCnt):
            section = uByteList[i]
            resultInt_LF = resultInt_LF + section*(256**(i))
        return resultInt_LF
    elif Spi_DataReturnMode == 'list':
        return uByteList
    else:
        jft.printToStdErr("Error, ResultMode must be 'list' or 'int_LF', or 'int_MF', not ",Spi_DataReturnMode)
        return []

        
SPI_helpText += "RemoveInterface removes the SPI Interface and all allocated resources of a SPI\n" \
                "Interface previously created with a successful call to the\n" \
                "CreateInterface() function.\n" \
                "Parameters:\n" \
                " SpiInterface  The name of any SpiInterface created by CreateInterface().\n" \
                "Return values:\n" \
                " 0             Success\n" \
                "               Any other value interrupts the execution and exits with an error message\n" \
                "   \n"

def RemoveInterface( SpiInterface ):
    retVal = jftserial1.JftSpi_RemoveInterface(to_char_p(SpiInterface) )
    jft.ProcessReturnValue(retVal)
    return retVal

SPI_helpText += "****************  End of SPI Help text  *******************\n"

